/**********************************************************************
Copyright (c) 2008 Andy Jefferson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
2008 Stefan Seelmann - connection pooling "JNDI"
    ...
 **********************************************************************/
package org.datanucleus.store.ldap;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.ldap.InitialLdapContext;
import javax.naming.ldap.LdapContext;
import javax.transaction.xa.XAResource;

import org.datanucleus.exceptions.NucleusException;
import org.datanucleus.metadata.AbstractClassMetaData;
import org.datanucleus.metadata.AbstractMemberMetaData;
import org.datanucleus.metadata.MetaDataManager;
import org.datanucleus.store.StoreManager;
import org.datanucleus.store.connection.AbstractConnectionFactory;
import org.datanucleus.store.connection.AbstractManagedConnection;
import org.datanucleus.store.connection.ManagedConnection;
import org.datanucleus.store.connection.ManagedConnectionResourceListener;

/**
 * Implementation of a ConnectionFactory for LDAP.
 */
public class ConnectionFactoryImpl extends AbstractConnectionFactory
{
    /**
     * Constructor.
     * @param storeMgr Store Manager
     * @param resourceType Type of resource (tx, nontx)
     */
    public ConnectionFactoryImpl(StoreManager storeMgr, String resourceType)
    {
        super(storeMgr, resourceType);
    }

    /**
     * Obtain a connection from the Factory. The connection will be enlisted within the {@link org.datanucleus.Transaction} 
     * associated to the <code>poolKey</code> if "enlist" is set to true.
     * @param poolKey the pool that is bound the connection during its lifecycle (or null)
     * @param options Any options for then creating the connection
     * @return the {@link org.datanucleus.store.connection.ManagedConnection}
     */
    public ManagedConnection createManagedConnection(Object poolKey, Map transactionOptions)
    {
        return new ManagedConnectionImpl(transactionOptions);
    }

    /**
     * Implementation of a ManagedConnection for LDAP.
     */
    public class ManagedConnectionImpl extends AbstractManagedConnection
    {
        Map options;

        LdapContext mainContext;

        List<LdapContext> subContexts;

        public ManagedConnectionImpl(Map options)
        {
            this.options = options;
        }

        public void close()
        {
            if (mainContext == null)
            {
                return;
            }
            for (ManagedConnectionResourceListener listener : listeners)
            {
                listener.managedConnectionPreClose();
            }
            try
            {
                try
                {
                    for (LdapContext subContext : subContexts)
                    {
                        subContext.close();
                    }
                    mainContext.close();
                }
                catch (Exception e)
                {
                    throw new NucleusException(e.getMessage(), e);
                }

            }
            finally
            {
                subContexts = null;
                mainContext = null;
                for (ManagedConnectionResourceListener listener : listeners)
                {
                    listener.managedConnectionPostClose();
                }
            }
        }

        public Object getConnection()
        {
            if (mainContext == null)
            {
                try
                {
                    Hashtable env = new Hashtable();
                    env.put(Context.INITIAL_CONTEXT_FACTORY, storeMgr.getConnectionDriverName());
                    env.put(Context.PROVIDER_URL, storeMgr.getConnectionURL());
                    env.put(Context.SECURITY_PRINCIPAL, storeMgr.getConnectionUserName());
                    env.put(Context.SECURITY_CREDENTIALS, storeMgr.getConnectionPassword());
                    if ("JNDI".equals(storeMgr.getStringProperty("datanucleus.connectionPoolingType")))
                    {
                        env.put("com.sun.jndi.ldap.connect.pool", "true");
                        // System.setProperty("com.sun.jndi.ldap.connect.pool.debug", "fine");

                        if (storeMgr.hasProperty("datanucleus.connectionPool.maxPoolSize"))
                        {
                            int size = storeMgr.getIntProperty("datanucleus.connectionPool.maxPoolSize");
                            if (size >= 0)
                            {
                                System.setProperty("com.sun.jndi.ldap.connect.pool.maxsize", "" + size);
                            }
                        }
                        if (storeMgr.hasProperty("datanucleus.connectionPool.initialPoolSize"))
                        {
                            int size = storeMgr.getIntProperty("datanucleus.connectionPool.initialPoolSize");
                            if (size >= 0)
                            {
                                System.setProperty("com.sun.jndi.ldap.connect.pool.initsize", "" + size);
                            }
                        }
                    }

                    // binary attributes
                    MetaDataManager mdm = storeMgr.getNucleusContext().getMetaDataManager();
                    Collection<String> classesWithMetaData = mdm.getClassesWithMetaData();
                    StringBuilder sb = new StringBuilder();
                    for (String className : classesWithMetaData)
                    {
                        AbstractClassMetaData cmd = mdm.getMetaDataForClass(className, storeMgr.getNucleusContext().getClassLoaderResolver(null));
                        List<AbstractMemberMetaData> mmds = LDAPUtils.getAllMemberMetaData(cmd);
                        for (AbstractMemberMetaData mmd : mmds)
                        {
                            if (mmd.getJoinMetaData() == null && mmd.getType().isArray())
                            {
                                String attributeNameForField = LDAPUtils.getAttributeNameForField(mmd);
                                sb.append(attributeNameForField).append(' ');
                            }
                        }
                    }
                    if (sb.length() > 0)
                    {
                        env.put("java.naming.ldap.attributes.binary", sb.toString());
                    }

                    mainContext = new InitialLdapContext(env, null);
                    subContexts = new ArrayList<LdapContext>();
                }
                catch (NamingException e)
                {
                    throw new NucleusException(e.getMessage(), e);
                }
            }

            try
            {
                // create a new instance of the main context to ensure thread safety
                LdapContext subContext = mainContext.newInstance(null);
                subContexts.add(subContext);
                return subContext;
            }
            catch (NamingException e)
            {
                throw new NucleusException(e.getMessage(), e);
            }
        }

        public XAResource getXAResource()
        {
            return null;
        }
    }
}